#!/usr/bin/env python2

import sys
import os
import subprocess
import argparse

arg_parser = argparse.ArgumentParser(description=
    "Changes RPATH in all ELF files in <path> (and its' subdirectories)"
    " to contain <library_path>")
arg_parser.add_argument("-l", "--list", action="store_true",
                      help="Only print current rpaths")
arg_parser.add_argument("-d", "--dry", action="store_true",
                      help="Dry run. Only print old and new rpaths, don't do anything")
arg_parser.add_argument("-c", "--confirm", action="store_true",
                      help="Ask before each rpath change")
arg_parser.add_argument("path", type=str,
                      help="The path to process")
arg_parser.add_argument("library_path", type=str,
                      help="The path to be added to the rpath")
args = arg_parser.parse_args()


if ((args.list and (args.dry or args.confirm)) or
    (args.dry and (args.list or args.confirm)) or
    (args.confirm and (args.list or args.dry))):
    sys.stderr.write("Arguments -l, -d and -c are mutually exclusive!\n")
    sys.exit(2)

MODE = "DEFAULT"
if args.list:
    MODE = "LIST"
elif args.dry:
    MODE = "DRY"
elif args.confirm:
    MODE = "CONFIRM"

top_dir = args.path
library_path = args.library_path

F_NULL = open("/dev/null", "w")

try:
    subprocess.call("patchelf", stdout = F_NULL, stderr = F_NULL)
except OSError:
    print("patchelf not found (or not working)")
    sys.exit(2)

def run_chrpath(abs_path):
    process = subprocess.Popen(['patchelf', '--print-rpath', abs_path], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = process.communicate()
    if process.returncode != 0:
        # probably not an ELF file, skip
        # print(err)
        return False

    rpath = out.strip()

    if  MODE == "LIST":
        print(abs_path)
        print("\t    RPATH: %s" % rpath)
        return True

    if rpath == "":
        rpaths = []
    else:
        rpaths = rpath.split(":")
    if library_path in rpaths:
        rpaths.remove(library_path)
    rpaths.insert(0, library_path)
    new_rpath = ":".join(rpaths)
    if rpath != new_rpath:
        print(abs_path)
        print("\t    RPATH: %s" % rpath)
        print("\tNEW RPATH: %s" % new_rpath)
        if MODE == "DRY":
            pass
        elif MODE == "CONFIRM":
            confirm = None
            while True:
                sys.stdout.write("\t    change? (y/n) ")
                confirm = sys.stdin.readline().strip()
                if confirm == "y" or confirm == "n":
                    break
            if confirm == "y":
                subprocess.call(['patchelf', '--set-rpath', new_rpath, abs_path])
        elif MODE == "DEFAULT":
            subprocess.call(['patchelf', '--set-rpath', new_rpath, abs_path])

    return True

if os.path.isdir(top_dir):
    for dirpath, dirnames, filenames in os.walk(top_dir):
        for filename in filenames:
            run_chrpath(os.path.abspath(os.path.join(dirpath, filename)))
else:
    run_chrpath(os.path.abspath(top_dir))
